/*
 *
 *    This file is a part of Space Geodetic Library. The library is used by
 *    nuSolve, a part of CALC/SOLVE system, and designed to make analysis of
 *    geodetic VLBI observations.
 *    Copyright (C) 2010-2020 Sergei Bolotin.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */



#include <SgTidalUt1.h>


/*=====================================================================================================*/
//
// constants:
//
/*=====================================================================================================*/

// IERS TN36, pp. 109--110:
const SgUt1TidalTableEntry ut1Ttable_IersConv2010[] = 
{
  //
  //    Multiple of          DUT1
  // l  l' F  D OMEGA     sin    cos
  {{ 1, 0, 2, 2, 2},    -0.0235, 0.0   },
  {{ 2, 0, 2, 0, 1},    -0.0404, 0.0   },
  {{ 2, 0, 2, 0, 2},    -0.0987, 0.0   },
  {{ 0, 0, 2, 2, 1},    -0.0508, 0.0   },
  {{ 0, 0, 2, 2, 2},    -0.1231, 0.0   },
  {{ 1, 0, 2, 0, 0},    -0.0385, 0.0   },
  {{ 1, 0, 2, 0, 1},    -0.4108, 0.0   },
  {{ 1, 0, 2, 0, 2},    -0.9926, 0.0   },
  {{ 3, 0, 0, 0, 0},    -0.0179, 0.0   },
  {{-1, 0, 2, 2, 1},    -0.0818, 0.0   },
  {{-1, 0, 2, 2, 2},    -0.1974, 0.0   },
  {{ 1, 0, 0, 2, 0},    -0.0761, 0.0   },
  {{ 2, 0, 2,-2, 2},     0.0216, 0.0   },
  {{ 0, 1, 2, 0, 2},     0.0254, 0.0   },
  {{ 0, 0, 2, 0, 0},    -0.2989, 0.0   },
  {{ 0, 0, 2, 0, 1},    -3.1873, 0.2010},
  {{ 0, 0, 2, 0, 2},    -7.8468, 0.5320},
  {{ 2, 0, 0, 0,-1},     0.0216, 0.0   },
  {{ 2, 0, 0, 0, 0},    -0.3384, 0.0   },
  {{ 2, 0, 0, 0, 1},     0.0179, 0.0   },
  {{ 0,-1, 2, 0, 2},    -0.0244, 0.0   },
  {{ 0, 0, 0, 2,-1},     0.0470, 0.0   },
  {{ 0, 0, 0, 2, 0},    -0.7341, 0.0   },
  {{ 0, 0, 0, 2, 1},    -0.0526, 0.0   },
  {{ 0,-1, 0, 2, 0},    -0.0508, 0.0   },
  {{ 1, 0, 2,-2, 1},     0.0498, 0.0   },
  {{ 1, 0, 2,-2, 2},     0.1006, 0.0   },
  {{ 1, 1, 0, 0, 0},     0.0395, 0.0   },
  {{-1, 0, 2, 0, 0},     0.0470, 0.0   },
  {{-1, 0, 2, 0, 1},     0.1767, 0.0   },
  {{-1, 0, 2, 0, 2},     0.4352, 0.0   },
  {{ 1, 0, 0, 0,-1},     0.5339, 0.0   },
  {{ 1, 0, 0, 0, 0},    -8.4046, 0.2500},
  {{ 1, 0, 0, 0, 1},     0.5443, 0.0   },
  {{ 0, 0, 0, 1, 0},     0.0470, 0.0   },
  {{ 1,-1, 0, 0, 0},    -0.0555, 0.0   },
  {{-1, 0, 0, 2,-1},     0.1175, 0.0   },
  {{-1, 0, 0, 2, 0},    -1.8236, 0.0   },
  {{-1, 0, 0, 2, 1},     0.1316, 0.0   },
  {{ 1, 0,-2, 2,-1},     0.0179, 0.0   },
  {{-1,-1, 0, 2, 0},    -0.0855, 0.0   },
  {{ 0, 2, 2,-2, 2},    -0.0573, 0.0   },
  {{ 0, 1, 2,-2, 1},     0.0329, 0.0   },
  {{ 0, 1, 2,-2, 2},    -1.8847, 0.0   },
  {{ 0, 0, 2,-2, 0},     0.2510, 0.0   },
  {{ 0, 0, 2,-2, 1},     1.1703, 0.0   },
  {{ 0, 0, 2,-2, 2},   -49.7174, 0.4330},
  {{ 0, 2, 0, 0, 0},    -0.1936, 0.0   },
  {{ 2, 0, 0,-2,-1},     0.0489, 0.0   },
  {{ 2, 0, 0,-2, 0},    -0.5471, 0.0   },
  {{ 2, 0, 0,-2, 1},     0.0367, 0.0   },
  {{ 0,-1, 2,-2, 1},    -0.0451, 0.0   },
  {{ 0, 1, 0, 0,-1},     0.0921, 0.0   },
  {{ 0,-1, 2,-2, 2},     0.8281, 0.0   },
  {{ 0, 1, 0, 0, 0},   -15.8887, 0.1530},
  {{ 0, 1, 0, 0, 1},    -0.1382, 0.0   },
  {{ 1, 0, 0,-1, 0},     0.0348, 0.0   },
  {{ 2, 0,-2, 0, 0},    -0.1372, 0.0   },
  {{-2, 0, 2, 0, 1},     0.4211, 0.0   },
  {{-1, 1, 0, 1, 0},    -0.0404, 0.0   },
  {{ 0, 0, 0, 0, 2},     7.8998, 0.0   },
  {{ 0, 0, 0, 0, 1}, -1617.2681, 0.0   },
};
const int numOfUt1TableRecs_IersConv2010 = sizeof(ut1Ttable_IersConv2010)/sizeof(SgUt1TidalTableEntry);




// IERS TN32, pp. 93--94:
const SgUt1TidalTableEntry ut1Ttable_IersConv2003[] = 
{
  //
  //    Multiple of          DUT1
  // l  l' F  D OMEGA     sin    cos
  {{ 1, 0, 2, 2, 2},    -0.02,  0.0 },
  {{ 2, 0, 2, 0, 1},    -0.04,  0.0 },
  {{ 2, 0, 2, 0, 2},    -0.10,  0.0 },
  {{ 0, 0, 2, 2, 1},    -0.05,  0.0 },
  {{ 0, 0, 2, 2, 2},    -0.12,  0.0 },
  {{ 1, 0, 2, 0, 0},    -0.04,  0.0 },
  {{ 1, 0, 2, 0, 1},    -0.41,  0.0 },
  {{ 1, 0, 2, 0, 2},    -1.00,  0.01},
  {{ 3, 0, 0, 0, 0},    -0.02,  0.0 },
  {{-1, 0, 2, 2, 1},    -0.08,  0.0 },
  {{-1, 0, 2, 2, 2},    -0.20,  0.0 },
  {{ 1, 0, 0, 2, 0},    -0.08,  0.0 },
  {{ 2, 0, 2,-2, 2},     0.02,  0.0 },
  {{ 0, 1, 2, 0, 2},     0.03,  0.0 },
  {{ 0, 0, 2, 0, 0},    -0.30,  0.0 },
  {{ 0, 0, 2, 0, 1},    -3.22,  0.02},
  {{ 0, 0, 2, 0, 2},    -7.79,  0.05},
  {{ 2, 0, 0, 0,-1},     0.02,  0.0 },
  {{ 2, 0, 0, 0, 0},    -0.34,  0.0 },
  {{ 2, 0, 0, 0, 1},     0.02,  0.0 },
  {{ 0,-1, 2, 0, 2},    -0.02,  0.0 },
  {{ 0, 0, 0, 2,-1},     0.05,  0.0 },
  {{ 0, 0, 0, 2, 0},    -0.74,  0.0 },
  {{ 0, 0, 0, 2, 1},    -0.05,  0.0 },
  {{ 0,-1, 0, 2, 0},    -0.05,  0.0 },
  {{ 1, 0, 2,-2, 1},     0.05,  0.0 },
  {{ 1, 0, 2,-2, 2},     0.10,  0.0 },
  {{ 1, 1, 0, 0, 0},     0.04,  0.0 },
  {{-1, 0, 2, 0, 0},     0.05,  0.0 },
  {{-1, 0, 2, 0, 1},     0.18,  0.0 },
  {{-1, 0, 2, 0, 2},     0.44,  0.0 },
  {{ 1, 0, 0, 0,-1},     0.54,  0.0 },
  {{ 1, 0, 0, 0, 0},    -8.33,  0.06},
  {{ 1, 0, 0, 0, 1},     0.55,  0.0 },
  {{ 0, 0, 0, 1, 0},     0.05,  0.0 },
  {{ 1,-1, 0, 0, 0},    -0.06,  0.0 },
  {{-1, 0, 0, 2,-1},     0.12,  0.0 },
  {{-1, 0, 0, 2, 0},    -1.84,  0.01},
  {{-1, 0, 0, 2, 1},     0.13,  0.0 },
  {{ 1, 0,-2, 2,-1},     0.02,  0.0 },
  {{-1,-1, 0, 2, 0},    -0.09,  0.0 },
  {{ 0, 2, 2,-2, 2},    -0.06,  0.0 },
  {{ 0, 1, 2,-2, 1},     0.03,  0.0 },
  {{ 0, 1, 2,-2, 2},    -1.91,  0.02},
  {{ 0, 0, 2,-2, 0},     0.26,  0.0 },
  {{ 0, 0, 2,-2, 1},     1.18, -0.01},
  {{ 0, 0, 2,-2, 2},   -49.06,  0.43},
  {{ 0, 2, 0, 0, 0},    -0.20,  0.0 },
  {{ 2, 0, 0,-2,-1},     0.05,  0.0 },
  {{ 2, 0, 0,-2, 0},    -0.56,  0.01},
  {{ 2, 0, 0,-2, 1},     0.04,  0.0 },
  {{ 0,-1, 2,-2, 1},    -0.05,  0.0 },
  {{ 0, 1, 0, 0,-1},     0.09,  0.0 },
  {{ 0,-1, 2,-2, 2},     0.82, -0.01},
  {{ 0, 1, 0, 0, 0},   -15.65,  0.15},
  {{ 0, 1, 0, 0, 1},    -0.14,  0.0 },
  {{ 1, 0, 0,-1, 0},     0.03,  0.0 },
  {{ 2, 0,-2, 0, 0},    -0.14,  0.0 },
  {{-2, 0, 2, 0, 1},     0.43, -0.01},
  {{-1, 1, 0, 1, 0},    -0.04,  0.0 },
  {{ 0, 0, 0, 0, 2},     8.20,  0.11},
  {{ 0, 0, 0, 0, 1}, -1689.54,-25.04},
};
const int numOfUt1TableRecs_IersConv2003 = sizeof(ut1Ttable_IersConv2003)/sizeof(SgUt1TidalTableEntry);



// IERS TN21, pp. 75--76:
const SgUt1TidalTableEntry ut1Ttable_IersConv1996[] = 
{
  //
  //    Multiple of          DUT1
  // l  l' F  D OMEGA     sin    cos
  {{ 1, 0, 2, 2, 2},    -0.02, 0.0 },
  {{ 2, 0, 2, 0, 1},    -0.04, 0.0 },
  {{ 2, 0, 2, 0, 2},    -0.10, 0.0 },
  {{ 0, 0, 2, 2, 1},    -0.05, 0.0 },
  {{ 0, 0, 2, 2, 2},    -0.12, 0.0 },
  {{ 1, 0, 2, 0, 0},    -0.04, 0.0 },
  {{ 1, 0, 2, 0, 1},    -0.40, 0.01},
  {{ 1, 0, 2, 0, 2},    -0.98, 0.03},
  {{ 3, 0, 0, 0, 0},    -0.02, 0.0 },
  {{ 1, 0, 2, 2, 1},    -0.08, 0.0 },
  {{ 1, 0, 2, 2, 2},    -0.20, 0.0 },
  {{ 1, 0, 0, 2, 0},    -0.08, 0.0 },
  {{ 2, 0, 2,-2, 2},     0.02, 0.0 },
  {{ 0, 1, 2, 0, 2},     0.03, 0.0 },
  {{ 0, 0, 2, 0, 0},    -0.30, 0.0 },
  {{ 0, 0, 2, 0, 1},    -3.20, 0.09},
  {{ 0, 0, 2, 0, 2},    -7.73, 0.21},
  {{ 2, 0, 0, 0,-1},     0.02, 0.0 },
  {{ 2, 0, 0, 0, 0},    -0.34, 0.0 },
  {{ 2, 0, 0, 0, 1},     0.02, 0.0 },
  {{ 0,-1, 2, 0, 2},    -0.02, 0.0 },
  {{ 0, 0, 0, 2,-1},     0.05, 0.0 },
  {{ 0, 0, 0, 2, 0},    -0.72, 0.02},
  {{ 0, 0, 0, 2, 1},    -0.05, 0.0 },
  {{ 0,-1, 0, 2, 0},    -0.05, 0.0 },
  {{ 1, 0, 2,-2, 1},     0.05, 0.0 },
  {{ 1, 0, 2,-2, 2},     0.10, 0.0 },
  {{ 1, 1, 0, 0, 0},     0.04, 0.0 },
  {{-1, 0, 2, 0, 0},     0.05, 0.0 },
  {{-1, 0, 2, 0, 1},     0.18, 0.0 },
  {{-1, 0, 2, 0, 2},     0.44, 0.0 },
  {{ 1, 0, 0, 0,-1},     0.53, 0.0 },
  {{ 1, 0, 0, 0, 0},    -8.33, 0.12},
  {{ 1, 0, 0, 0, 1},     0.54, 0.0 },
  {{ 0, 0, 0, 1, 0},     0.05, 0.0 },
  {{ 1,-1, 0, 0, 0},    -0.06, 0.0 },
  {{-1, 0, 0, 2,-1},     0.12, 0.0 },
  {{-1, 0, 0, 2, 0},    -1.84, 0.02},
  {{-1, 0, 0, 2, 1},     0.13, 0.0 },
  {{ 1, 0,-2, 2,-1},     0.02, 0.0 },
  {{-1,-1, 0, 2, 0},    -0.09, 0.0 },
  {{ 0, 2, 2,-2, 2},    -0.06, 0.0 },
  {{ 0, 1, 2,-2, 1},     0.03, 0.0 },
  {{ 0, 1, 2,-2, 2},    -1.88, 0.0 },
  {{ 0, 0, 2,-2, 0},     0.25, 0.0 },
  {{ 0, 0, 2,-2, 1},     1.17, 0.0 },
  {{ 0, 0, 2,-2, 2},   -48.84, 0.11},
  {{ 0, 2, 0, 0, 0},    -0.19, 0.0 },
  {{ 2, 0, 0,-2,-1},     0.05, 0.0 },
  {{ 2, 0, 0,-2, 0},    -0.55, 0.0 },
  {{ 2, 0, 0,-2, 1},     0.04, 0.0 },
  {{ 0,-1, 2,-2, 1},    -0.05, 0.0 },
  {{ 0, 1, 0, 0,-1},     0.09, 0.0 },
  {{ 0,-1, 2,-2, 2},     0.83, 0.0 },
  {{ 0, 1, 0, 0, 0},   -15.55, 0.02},
  {{ 0, 1, 0, 0, 1},    -0.14, 0.0 },
  {{ 1, 0, 0,-1, 0},     0.03, 0.0 },
  {{ 2, 0,-2, 0, 0},    -0.14, 0.0 },
  {{-2, 0, 2, 0, 1},     0.42, 0.0 },
  {{-1, 1, 0, 1, 0},     0.04, 0.0 },
  {{ 0, 0, 0, 0, 2},     7.90, 0.0 },
  {{ 0, 0, 0, 0, 1}, -1637.68,-0.10},
};
const int numOfUt1TableRecs_IersConv1996 = sizeof(ut1Ttable_IersConv1996)/sizeof(SgUt1TidalTableEntry);



// IERS TN13, pp. 114--115:
const SgUt1TidalTableEntry ut1Ttable_IersStds1992[] = 
{
  //
  //    Multiple of          DUT1
  // l  l' F  D OMEGA     sin    cos
  {{ 1, 0, 2, 2, 2},    -0.02, 0.0 },
  {{ 2, 0, 2, 0, 1},    -0.04, 0.0 },
  {{ 2, 0, 2, 0, 2},    -0.10, 0.0 },
  {{ 0, 0, 2, 2, 1},    -0.05, 0.0 },
  {{ 0, 0, 2, 2, 2},    -0.12, 0.0 },
  {{ 1, 0, 2, 0, 0},    -0.04, 0.0 },
  {{ 1, 0, 2, 0, 1},    -0.40, 0.01},
  {{ 1, 0, 2, 0, 2},    -0.98, 0.03},
  {{ 3, 0, 0, 0, 0},    -0.02, 0.0 },
  {{-1, 0, 2, 2, 1},    -0.08, 0.0 },
  {{-1, 0, 2, 2, 2},    -0.20, 0.0 },
  {{ 1, 0, 0, 2, 0},    -0.08, 0.0 },
  {{ 2, 0, 2,-2, 2},     0.02, 0.0 },
  {{ 0, 1, 2, 0, 2},     0.03, 0.0 },
  {{ 0, 0, 2, 0, 0},    -0.30, 0.0 },
  {{ 0, 0, 2, 0, 1},    -3.20, 0.09},
  {{ 0, 0, 2, 0, 2},    -7.73, 0.21},
  {{ 2, 0, 0, 0,-1},     0.02, 0.0 },
  {{ 2, 0, 0, 0, 0},    -0.34, 0.0 },
  {{ 2, 0, 0, 0, 1},     0.02, 0.0 },
  {{ 0,-1, 2, 0, 2},    -0.02, 0.0 },
  {{ 0, 0, 0, 2,-1},     0.05, 0.0 },
  {{ 0, 0, 0, 2, 0},    -0.72, 0.02},
  {{ 0, 0, 0, 2, 1},    -0.05, 0.0 },
  {{ 0,-1, 0, 2, 0},    -0.05, 0.0 },
  {{ 1, 0, 2,-2, 1},     0.05, 0.0 },
  {{ 1, 0, 2,-2, 2},     0.10, 0.0 },
  {{ 1, 1, 0, 0, 0},     0.04, 0.0 },
  {{-1, 0, 2, 0, 0},     0.05, 0.0 },
  {{-1, 0, 2, 0, 1},     0.18, 0.0 },
  {{-1, 0, 2, 0, 2},     0.44, 0.0 },
  {{ 1, 0, 0, 0,-1},     0.53, 0.0 },
  {{ 1, 0, 0, 0, 0},    -8.33, 0.12},
  {{ 1, 0, 0, 0, 1},     0.54, 0.0 },
  {{ 0, 0, 0, 1, 0},     0.05, 0.0 },
  {{ 1,-1, 0, 0, 0},    -0.06, 0.0 },
  {{-1, 0, 0, 2,-1},     0.12, 0.0 },
  {{-1, 0, 0, 2, 0},    -1.84, 0.02},
  {{-1, 0, 0, 2, 1},     0.13, 0.0 },
  {{ 1, 0,-2, 2,-1},     0.02, 0.0 },
  {{-1,-1, 0, 2, 0},    -0.09, 0.0 },
  {{ 0, 2, 2,-2, 2},    -0.06, 0.0 },
  {{ 0, 1, 2,-2, 1},     0.03, 0.0 },
  {{ 0, 1, 2,-2, 2},    -1.88, 0.0 },
  {{ 0, 0, 2,-2, 0},     0.25, 0.0 },
  {{ 0, 0, 2,-2, 1},     1.17, 0.0 },
  {{ 0, 0, 2,-2, 2},   -48.84, 0.11},
  {{ 0, 2, 0, 0, 0},    -0.19, 0.0 },
  {{ 2, 0, 0,-2,-1},     0.05, 0.0 },
  {{ 2, 0, 0,-2, 0},    -0.55, 0.0 },
  {{ 2, 0, 0,-2, 1},     0.04, 0.0 },
  {{ 0,-1, 2,-2, 1},    -0.05, 0.0 },
  {{ 0, 1, 0, 0,-1},     0.09, 0.0 },
  {{ 0,-1, 2,-2, 2},     0.83, 0.0 },
  {{ 0, 1, 0, 0, 0},   -15.55, 0.02},
  {{ 0, 1, 0, 0, 1},    -0.14, 0.0 },
  {{ 1, 0, 0,-1, 0},     0.03, 0.0 },
  {{ 2, 0,-2, 0, 0},    -0.14, 0.0 },
  {{-2, 0, 2, 0, 1},     0.42, 0.0 },
  {{-1, 1, 0, 1, 0},     0.04, 0.0 },
  {{ 0, 0, 0, 0, 2},     7.90, 0.0 },
  {{ 0, 0, 0, 0, 1}, -1637.68, 0.10},
};
const int numOfUt1TableRecs_IersStds1992 = sizeof(ut1Ttable_IersStds1992)/sizeof(SgUt1TidalTableEntry);
/*=====================================================================================================*/











/*=====================================================================================================*/
